@@ -44,9 +44,16 @@ def JoinPart(group_by: api.GroupBy,
         if '__name__' in ref and ref['__name__'].startswith("group_bys"):
             group_by_module_name = ref['__name__']
             break
-    logging.debug("group_by's module info from garbage collector {}".format(group_by_module_name))
-    group_by_module = importlib.import_module(group_by_module_name)
-    __builtins__['__import__'] = eo.import_module_set_name(group_by_module, api.GroupBy)
+    if group_by_module_name:
+        logging.debug("group_by's module info from garbage collector {}".format(group_by_module_name))
+        group_by_module = importlib.import_module(group_by_module_name)
+        __builtins__['__import__'] = eo.import_module_set_name(group_by_module, api.GroupBy)
+    else:
+        if not group_by.metaData.name:
+            logging.error("No group_by file or custom group_by name found")
+            raise ValueError(
+                "[GroupBy] Must specify a group_by name if group_by is not defined in separate file. "
+                "You may pass it in via GroupBy.name. \n")
     if key_mapping:
         utils.check_contains(key_mapping.values(),
                              group_by.keyColumns,
@@ -211,6 +218,89 @@ def ExternalPart(source: api.ExternalSource,
     )
 
 
+def LabelPart(labels: List[api.JoinPart],
+              left_start_offset: int,
+              left_end_offset: int,
+              label_offline_schedule: str = '@daily') -> api.LabelPart:
+    """
+    Used to describe labels in join. Label part can be viewed as regular join part but represent
+    label data instead of regular feature data. Once labels are mature, label join job would join
+    labels with features in the training window user specified using `leftStartOffset` and
+    `leftEndOffset`.
+
+    Labels will be refreshed within this window given a label ds. As a result, there could be multiple
+    label versions based on the label ds. Label definition can be updated along the way but label join
+    job can only accommodate the changes going forward unless a backfill is manually triggered
+
+    :param labels: List of labels
+    :param left_start_offset: Integer to define the earliest date label should be refreshed
+                            comparing to label_ds date specified
+    :param left_end_offset: Integer to define the most recent date label should be refreshed.
+                          e.g. left_end_offset = 3 most recent label available will be 3 days
+                          prior to 'label_ds'
+    :param label_offline_schedule: Cron expression for Airflow to schedule a DAG for offline
+                                   label join compute tasks
+    """
+
+    label_metadata = api.MetaData(
+        offlineSchedule=label_offline_schedule
+    )
+
+    return api.LabelPart(
+        labels=labels,
+        leftStartOffset=left_start_offset,
+        leftEndOffset=left_end_offset,
+        metaData=label_metadata
+    )
+
+
+def Derivation(name: str, expression: str) -> api.Derivation:
+    """
+    Derivation allows arbitrary SQL select clauses to be computed using columns from joinPart and externalParts,
+    and saves the result as derived columns. The results will be available both in online fetching response map,
+    and in offline Hive table.
+
+    joinPart column names are automatically constructed according to the below convention
+    `{join_part_prefix}_{group_by_name}_{input_column_name}_{aggregation_operation}_{window}_{by_bucket}`
+    prefix, window and bucket are optional. You can find the type information of columns using the analyzer tool.
+
+    externalPart column names are automatically constructed according to the below convention
+    `ext_{external_source_name}_{value_column}`.
+    Types are defined along with the schema by users for external sources.
+
+    Furthermore, since contextualSource is a special type of externalSource, any contextual columns can be referenced
+    in the SQL either in their "key" format (without prefix), or in their value format (with prefix `ext_contextual_`).
+
+    If both name and expression are set to "*", then every raw column will be included along with the derived columns.
+
+    :param name: output column name of the SQL expression
+    :param expression: any valid Spark SQL select clause based on joinPart or externalPart columns
+    :return: a Derivation object representing a single derived column or a wildcard ("*") selection.
+    """
+    return api.Derivation(
+        name=name, expression=expression
+    )
+
+
+def BootstrapPart(table: str, key_columns: List[str] = None, query: api.Query = None) -> api.BootstrapPart:
+    """
+    Bootstrap is the concept of using pre-computed feature values and skipping backfill computation during the
+    training data generation phase. Bootstrap can be used for many purposes:
+    - Generating ongoing feature values from logs
+    - Backfilling feature values for external features (in which case Chronon is unable to run backfill)
+    - Initializing a new Join by migrating old data from an older Join and reusing data
+
+    :param table: Name of hive table that contains feature values where rows are 1:1 mapped to left table
+    :param key_columns: Keys to join bootstrap table to left table
+    :param query: Selected columns (features & keys) and filtering conditions of the bootstrap tables.
+    """
+    return api.BootstrapPart(
+        table=table,
+        query=query,
+        keyColumns=key_columns
+    )
+
+
 def Join(left: api.Source,
          right_parts: List[api.JoinPart],
          check_consistency: bool = False,
@@ -224,9 +314,15 @@ def Join(left: api.Source,
          env: Dict[str, Dict[str, str]] = None,
          lag: int = 0,
          skew_keys: Dict[str, List[str]] = None,
-         sample_percent: float = None,  # will sample all the requests based on sample percent
+         sample_percent: float = 100.0,
+         consistency_sample_percent: float = 5.0,
          online_external_parts: List[api.ExternalPart] = None,
          offline_schedule: str = '@daily',
+         row_ids: List[str] = None,
+         bootstrap_parts: List[api.BootstrapPart] = None,
+         bootstrap_from_log: bool = False,
+         label_part: api.LabelPart = None,
+         derivations: List[api.Derivation] = None,
          **kwargs
          ) -> api.Join:
     """
@@ -290,11 +386,25 @@ def Join(left: api.Source,
         This is used to blacklist crawlers etc
     :param sample_percent:
         Online only parameter. What percent of online serving requests to this join should be logged into warehouse.
+    :param consistency_sample_percent:
+        Online only parameter. What percent of online serving requests to this join should be sampled to compute
+        online offline consistency metrics.
+        if sample_percent=50.0 and consistency_sample_percent=10.0, then basically the consistency job runs on
+        5% of total traffic.
     :param online_external_parts:
         users can register external sources into Api implementation. Chronon fetcher can invoke the implementation.
         This is applicable only for online fetching. Offline this will not be produce any values.
     :param offline_schedule:
         Cron expression for Airflow to schedule a DAG for offline join compute tasks
+    :param row_ids:
+        Columns of the left table that uniquely define a training record. Used as default keys during bootstrap
+    :param bootstrap_parts:
+        A list of BootstrapPart used for the Join. See BootstrapPart doc for more details
+    :param bootstrap_from_log:
+        If set to True, will use logging table to generate training data by default and skip continuous backfill.
+        Logging will be treated as another bootstrap source, but other bootstrap_parts will take precedence.
+    :param label_part:
+        Label part which contains a list of labels and label refresh window boundary used for the Join
     :return:
         A join object that can be used to backfill or serve data. For ML use-cases this should map 1:1 to model.
     """
@@ -328,6 +438,18 @@ def Join(left: api.Source,
     right_dependencies = [dep for (source, meta_data) in right_info for dep in
                           utils.get_dependencies(source, dependencies, meta_data, lag=lag)]
 
+    if label_part:
+        label_dependencies = utils.get_label_table_dependencies(label_part)
+        label_metadata = api.MetaData(
+            dependencies=utils.dedupe_in_order(left_dependencies + label_dependencies),
+            offlineSchedule=label_part.metaData.offlineSchedule
+        )
+        label_part = api.LabelPart(
+            labels=label_part.labels,
+            leftStartOffset=label_part.leftStartOffset,
+            leftEndOffset=label_part.leftEndOffset,
+            metaData=label_metadata)
+
     custom_json = {
         "check_consistency": check_consistency,
         "lag": lag
@@ -340,6 +462,8 @@ def Join(left: api.Source,
         custom_json["additional_env"] = additional_env
     custom_json.update(kwargs)
 
+    consistency_sample_percent = consistency_sample_percent if check_consistency else None
+
     metadata = api.MetaData(
         online=online,
         production=production,
@@ -349,7 +473,8 @@ def Join(left: api.Source,
         tableProperties=table_properties,
         modeToEnvMap=env,
         samplePercent=sample_percent,
-        offlineSchedule=offline_schedule
+        offlineSchedule=offline_schedule,
+        consistencySamplePercent=consistency_sample_percent
     )
 
     # external parts need to be unique on (prefix, part.source.metaData.name)
@@ -362,10 +487,24 @@ def Join(left: api.Source,
                 print(f"Found {count - 1} duplicate(s) for external part {key}")
         assert has_duplicates is False, "Please address all the above mentioned duplicates."
 
+    if bootstrap_from_log:
+        has_logging = sample_percent > 0 and online
+        assert has_logging, "Join must be online with sample_percent set in order to use bootstrap_from_log option"
+        bootstrap_parts = (bootstrap_parts or []) + [
+            api.BootstrapPart(
+                # templated values will be replaced when metaData.name is set at the end
+                table="{{ logged_table }}"
+            )
+        ]
+
     return api.Join(
         left=updated_left,
         joinParts=right_parts,
         metaData=metadata,
         skewKeys=skew_keys,
-        onlineExternalParts=online_external_parts
+        onlineExternalParts=online_external_parts,
+        bootstrapParts=bootstrap_parts,
+        rowIds=row_ids,
+        labelPart=label_part,
+        derivations=derivations
     )
