@@ -16,13 +16,14 @@ ONLINE_WRITE_ARGS = '--conf-path={conf_path} ' + ONLINE_ARGS
 ONLINE_OFFLINE_WRITE_ARGS = OFFLINE_ARGS + ONLINE_ARGS
 ONLINE_MODES = ['streaming', 'metadata-upload', 'fetch', 'local-streaming']
 SPARK_MODES = ['backfill', 'upload', 'streaming', 'consistency-metrics-compute',
-               'analyze', 'stats-summary', 'log-flattener', 'metadata-export']
+               'analyze', 'stats-summary', 'log-flattener', 'metadata-export', 'label-join']
 
 # Constants for supporting multiple spark versions.
-SUPPORTED_SPARK = ['2.4.0', '3.1.1']
+SUPPORTED_SPARK = ['2.4.0', '3.1.1', '3.2.1']
 SCALA_VERSION_FOR_SPARK = {
     '2.4.0': "2.11",
     '3.1.1': "2.12",
+    '3.2.1': "2.13"
 }
 
 MODE_ARGS = {
@@ -37,6 +38,7 @@ MODE_ARGS = {
     'local-streaming': ONLINE_WRITE_ARGS + ' -d',
     'log-flattener': OFFLINE_ARGS,
     'metadata-export': OFFLINE_ARGS,
+    'label-join': OFFLINE_ARGS,
 }
 
 ROUTES = {
@@ -58,6 +60,7 @@ ROUTES = {
         'analyze': 'analyze',
         'log-flattener': 'log-flattener',
         'metadata-export': 'metadata-export',
+        'label-join': 'label-join',
     },
     'staging_queries': {
         'backfill': 'staging-query-backfill',
@@ -90,12 +93,12 @@ def retry_decorator(retries=3, backoff=20):
 
 def check_call(cmd):
     print("Running command: " + cmd)
-    return subprocess.check_call(cmd.split())
+    return subprocess.check_call(cmd.split(), bufsize=0)
 
 
 def check_output(cmd):
     print("Running command: " + cmd)
-    return subprocess.check_output(cmd.split()).strip()
+    return subprocess.check_output(cmd.split(), bufsize=0).strip()
 
 
 def download_only_once(url, path):
@@ -135,6 +138,8 @@ def download_jar(version, jar_type='uber', release_tag=None, spark_version="2.4.
     print("Downloading jar from url: " + base_url)
     jar_path = os.environ.get('CHRONON_JAR_PATH', None)
     if jar_path is None:
+        if version == 'latest':
+            version = None
         if version is None:
             metadata_content = check_output("curl -s {}/maven-metadata.xml".format(base_url))
             meta_tree = ET.fromstring(metadata_content)
@@ -299,7 +304,7 @@ if __name__ == "__main__":
     parser.add_argument('--online-class',
                         help='Class name of Online Impl. Used for streaming and metadata-upload mode.',
                         default=os.environ.get('CHRONON_ONLINE_CLASS', None))
-    parser.add_argument('--version', help='Chronon version to use.', default=None)
+    parser.add_argument('--version', help='Chronon version to use.', default=os.environ.get('VERSION', None)),
     parser.add_argument('--spark-version', help='Spark version to use for downloading jar.',
                         default=os.environ.get('SPARK_VERSION', '2.4.0')),
     parser.add_argument('--spark-submit-path',
